package items

import (
	"git.perx.ru/perxis/perxis-go/pkg/locales"
	"git.perx.ru/perxis/perxis-go/pkg/options"
)

type StorageFindOptions struct {
	Options
	options.FindOptions

	Deleted   bool
	Regular   bool
	Hidden    bool
	Templates bool

	// Язык перевода, который будет использоваться. Если не указан, то возвращаются данные для языка по умолчанию
	LocaleID string

	// Список идентификаторов переводов/локалей, которых должны быть включены в результат
	TranslationsIDs []string

	// Список локалей пространства
	Locales []*locales.Locale
}

func NewStorageFindOptions(opts ...interface{}) *StorageFindOptions {
	fo := &StorageFindOptions{}
	for _, o := range opts {
		switch o := o.(type) {
		case *FindOptions:
			fo.Options = o.Options
			fo.FindOptions = o.FindOptions
			fo.Deleted = o.Deleted
			fo.Regular = o.Regular
			fo.Hidden = o.Hidden
			fo.Templates = o.Templates
			fo.LocaleID = o.LocaleID
			fo.TranslationsIDs = o.TranslationsIDs
		case *FindPublishedOptions:
			fo.Options = o.Options
			fo.FindOptions = o.FindOptions
			fo.Deleted = o.Deleted
			fo.Regular = o.Regular
			fo.Hidden = o.Hidden
			fo.Templates = o.Templates
			fo.LocaleID = o.LocaleID
			fo.TranslationsIDs = o.TranslationsIDs
		case *FindArchivedOptions:
			fo.Options = o.Options
			fo.FindOptions = o.FindOptions
			fo.LocaleID = o.LocaleID
			fo.TranslationsIDs = o.TranslationsIDs
		case *GetRevisionOptions:
			fo.Options = o.Options
			fo.LocaleID = o.LocaleID
			fo.TranslationsIDs = o.TranslationsIDs
		case *ListRevisionsOptions:
			fo.Options = o.Options
			fo.FindOptions = o.FindOptions
			fo.LocaleID = o.LocaleID
			fo.TranslationsIDs = o.TranslationsIDs
		case *locales.Locale:
			fo.Locales = []*locales.Locale{o}
		case []*locales.Locale:
			fo.Locales = o
		}
	}
	return fo
}

func MergeStorageFindOptions(opts ...*StorageFindOptions) *StorageFindOptions {
	o := NewStorageFindOptions()
	for _, opt := range opts {
		if opt == nil {
			continue
		}

		o.Options = MergeOptions(o.Options, opt.Options)
		o.FindOptions = *options.MergeFindOptions(&o.FindOptions, &opt.FindOptions)

		o.Deleted = o.Deleted || opt.Deleted
		o.Regular = o.Regular || opt.Regular
		o.Templates = o.Templates || opt.Templates
		o.Hidden = o.Hidden || opt.Hidden

		if opt.LocaleID != "" {
			o.LocaleID = opt.LocaleID
		}
		o.TranslationsIDs = append(o.TranslationsIDs, opt.TranslationsIDs...)
		o.Locales = append(o.Locales, opt.Locales...)
	}
	return o
}
